import { world } from "@minecraft/server";
import { VERTICAL_SLAB_DATA } from "../../library/data/verticalSlabData";
import { DOUBLE_VERTICAL_SLAB_STATES } from "../../library/data/stateData";
import { VERTICAL_SLABS } from "../../library/enum/blockList";
import { CopperTransformationType } from "../../library/enum/system";
import { CUSTOM_BLOCK_TAGS } from "../../library/enum/tags";
import { Utils } from "../../library/utils/systemUtils";
import { BlockUtils } from "../../library/utils/blockUtils";
import { VerticalSlabUtils } from "../../library/utils/verticalSlabUtils";

export class VerticalSlab {
    #block;
    verticalSlab;

    constructor(block) {
        this.location = block.location;
        this.dimension = block.dimension.id;
        this.#block = block;
        for (const [type, value] of Object.entries(VERTICAL_SLABS)) {
            if(block.permutation.matches(value)) {
                this.id = value;
                this.data = VERTICAL_SLAB_DATA[type];
                this.verticalSlab = block;
                break;
            };
        };
    }

    get isVerticalSlab() {
        return VerticalSlabUtils.isVerticalSlab(this.#block);
    }

    get copperStage() { return this.data.copperStage; }
    get copperIsWaxed() { return this.data.isWaxed; }

    static checkIsVerticalSlab(id) {
        return Object.values(VERTICAL_SLABS).includes(id);
    }

    #findVerticalSlab(callback) {
        for (const [key, value] of Object.entries(VERTICAL_SLAB_DATA)) {
            if (callback(value)) return VERTICAL_SLABS[key];
        };
    }

    #loadStructure(name, dim, {x, y, z}, dir) {
        world.getDimension(dim).runCommand(`structure load ${name.slice(5)}:${dir} ${x} ${y} ${z}`);
    }

    #wax() {
        if (!this.copperIsWaxed) {
            this.transVerticalSlabType(this.#findVerticalSlab(vals => vals.copperStage == this.copperStage && vals.isWaxed == true));
        };
    }

    #scrape() {
        if (this.copperIsWaxed) { // Dewax
            this.transVerticalSlabType(this.#findVerticalSlab(vals => vals.copperStage == this.copperStage && vals.isWaxed == false));
        } else if (this.copperStage >= 1) { // Deoxidize
            this.transVerticalSlabType(this.#findVerticalSlab(vals => vals.copperStage == this.copperStage - 1 && vals.isWaxed == false));
        };
    }

    #oxidize() {
        if (this.copperStage < 3) {
            this.transVerticalSlabType(this.#findVerticalSlab(vals => vals.copperStage == this.copperStage + 1 && vals.isWaxed == false));
        };
    }

    getDirection() {
        return VerticalSlabUtils.getDirection(this.verticalSlab);
    }

    setStates(states = {}) {
        BlockUtils.setStates(this.verticalSlab, states);
    }

    transVerticalSlabType(newVerticalSlabId) {
        if (this.isVerticalSlab) {
            if (this.verticalSlab.isWaterlogged) {
                this.#loadStructure(newVerticalSlabId, this.dimension, this.location, this.getDirection());
            } else {
                BlockUtils.setBlock(this.dimension, this.location, newVerticalSlabId, this.verticalSlab.permutation.getAllStates());
            };
        };
    }

    merge() {
        if (this.isVerticalSlab) {
            const doubleVerticalSlab = VerticalSlabUtils.verticalSlabToDouble(this.id);
            let state = {};
            if (this.data.hasDirection) {
                state = BlockUtils.createStateRecord(DOUBLE_VERTICAL_SLAB_STATES, Utils.axis(this.getDirection()));
            };
            BlockUtils.setBlock(this.dimension, this.location, doubleVerticalSlab, state);
            return true;
        };
        return false;
    }

    waterlog() {
        if (this.isVerticalSlab) {
            this.#loadStructure(this.id, this.dimension, this.location, this.getDirection());
            return true;
        };
        return false;
    }

    transCopper(transType) {
        if (this.isVerticalSlab && this.verticalSlab.hasTag(CUSTOM_BLOCK_TAGS.Copper)) {
            switch(transType) {
                case CopperTransformationType.WAX: this.#wax();
                    break;
                case CopperTransformationType.SCRAPE: this.#scrape();
                    break;
                case CopperTransformationType.OXIDIZE: this.#oxidize();
                    break;
                default: this.#oxidize();
            };
        };
    }
}