import { world, ItemStack } from "@minecraft/server";
import { VERTICAL_SLABS, DOUBLE_VERTICAL_SLABS } from "../../../library/enum/blockList";
import { VANILLA_BLOCK_TAGS, VANILLA_ITEM_TAGS, CUSTOM_BLOCK_TAGS } from "../../../library/enum/tags";
import { Utils } from "../../../library/utils/systemUtils";
import { PlayerUtils } from "../../../library/utils/playerUtils";
import { VerticalSlabUtils } from "../../../library/utils/verticalSlabUtils";

const tier = {
    [VANILLA_ITEM_TAGS.WoodenTier]: VANILLA_BLOCK_TAGS.WoodPickDiggable,
    [VANILLA_ITEM_TAGS.StoneTier]: VANILLA_BLOCK_TAGS.StonePickDiggable,
    [VANILLA_ITEM_TAGS.IronTier]: VANILLA_BLOCK_TAGS.IronPickDiggable,
    [VANILLA_ITEM_TAGS.GoldenTier]: VANILLA_BLOCK_TAGS.GoldPickDiggable,
    [VANILLA_ITEM_TAGS.DiamondTier]: VANILLA_BLOCK_TAGS.DiamondPickDiggable,
    [VANILLA_ITEM_TAGS.NetheriteTier]: VANILLA_BLOCK_TAGS.DiamondPickDiggable
};

export class VerticalSlabBreakEvent {
    #isDoubleSlab;
    #blockTags;
    brokenBlockId;

    setup() {
        let playerIsCreativeMode = true;
        const availableBlock = {blockTypes: Object.values(VERTICAL_SLABS).concat(Object.values(DOUBLE_VERTICAL_SLABS))};
        world.beforeEvents.playerBreakBlock.subscribe(data => {
            if (!PlayerUtils.isCreativeMode(data.player)) {
                playerIsCreativeMode = false;
                this.#beforePlayerBreak(data);
            };
        }, availableBlock);
        world.afterEvents.playerBreakBlock.subscribe(data => {
            if (!playerIsCreativeMode) {
                this.#damageDurability(data);
                this.#loot(data);
            };
        }, availableBlock);
    }

    #beforePlayerBreak(data) {
        const block = data.block;
        this.#isDoubleSlab = block.hasTag(CUSTOM_BLOCK_TAGS.DoubleSlab);
        for (const value of Object.values(this.#isDoubleSlab ? DOUBLE_VERTICAL_SLABS : VERTICAL_SLABS)) {
            if (block.permutation.matches(value)) {
                this.brokenBlockId = value;
                break;
            };
        };
        this.#blockTags = block.getTags();
    }

    #damageDurability(data) {
        PlayerUtils.damageMainhandItem(data.player);
    }

    #loot(data) {
        if (this.brokenBlockId === undefined) {
            return;
        };
        let canDig = false;
        const isPickaxeDiggable = this.#blockTags.some(tag => Object.values(tier).includes(tag));
        if (isPickaxeDiggable) {
            const pickaxeTire = data.itemStackBeforeBreak?.getTags().filter(tag => Object.keys(tier).includes(tag));
            if (!pickaxeTire || pickaxeTire instanceof Array && pickaxeTire.length === 0) {
                return;
            };
            for (const tag of pickaxeTire) {
                if (this.#blockTags.includes(tier[tag])) {
                    canDig = true;
                };
            };
        };
        if (!isPickaxeDiggable || isPickaxeDiggable && canDig) {
            const lootItem = new ItemStack(this.#isDoubleSlab ? VerticalSlabUtils.doubleToVerticalSlab(this.brokenBlockId) : this.brokenBlockId, this.#isDoubleSlab ? 2 : 1);
            data.player.dimension.spawnItem(lootItem, Utils.transToWorldLoc(data.block.location));
        };
    }
}

export const verticalSlabBreak = new VerticalSlabBreakEvent();